@@ -120,7 +120,6 @@ inline void URI::guess_leading_authority () {
     // 2) if scheme is present and first path part is a valid port -> scheme is host, first path part is port.
     // otherwise leave parsed url unchanged as it has no leading authority
     if (!_scheme.length()) {
-        //_scheme = default_scheme;
         size_t delim = _path.find('/');
         if (delim == string::npos) {
             _host = _path;
@@ -175,12 +174,12 @@ void URI::parse (const string& uristr) {
         if (state == STATE_SCHEME && p[i] == ':') {                     // custom processing
             if (len > i + 2 && p[i+1] == '/' && p[i+2] == '/') {        // 'scheme://netloc' case
                 state = STATE_HOST;
-                _scheme.assign(p, i, string::COPY);
+                _scheme = uristr.substr(0, i);
                 i += 2;
                 marks[STATE_HOST].start = marks[STATE_UINFO].start = i + 1;
             } else {                                                    // 'scheme:path' case
                 state = STATE_PATH;
-                _scheme.assign(p, i, string::COPY);
+                _scheme = uristr.substr(0, i);
                 marks[STATE_PATH].start = i + 1;
             }
             continue;
@@ -195,10 +194,10 @@ void URI::parse (const string& uristr) {
     marks[parseinfo[state][0].seen_state].end = i;
 
     if (marks[STATE_UINFO].end > 0)
-        decode_uri_component(p + marks[STATE_UINFO].start, marks[STATE_UINFO].end - marks[STATE_UINFO].start, _user_info);
+        decode_uri_component(string_view(p + marks[STATE_UINFO].start, marks[STATE_UINFO].end - marks[STATE_UINFO].start), _user_info);
 
     if (marks[STATE_HOST].end > 0) {
-        decode_uri_component(p + marks[STATE_HOST].start, marks[STATE_HOST].end - marks[STATE_HOST].start, _host);
+        decode_uri_component(string_view(p + marks[STATE_HOST].start, marks[STATE_HOST].end - marks[STATE_HOST].start), _host);
         if (marks[STATE_PORT].end > 0) {
             const char* portp = p + marks[STATE_PORT].start;
             size_t len = marks[STATE_PORT].end - marks[STATE_PORT].start;
@@ -211,25 +210,19 @@ void URI::parse (const string& uristr) {
     }
 
     if (marks[STATE_PATH].end > 0)
-        _path.assign(p + marks[STATE_PATH].start, marks[STATE_PATH].end - marks[STATE_PATH].start, string::COPY);
+        _path = uristr.substr(marks[STATE_PATH].start, marks[STATE_PATH].end - marks[STATE_PATH].start);
     if (marks[STATE_QUERY].end > 0) { // assign as is, raw_query getter or parse_query will actually decode
-        _qstr.assign(p + marks[STATE_QUERY].start, marks[STATE_QUERY].end - marks[STATE_QUERY].start, string::COPY);
+        _qstr = uristr.substr(marks[STATE_QUERY].start, marks[STATE_QUERY].end - marks[STATE_QUERY].start);
         ok_qstr();
     }
     if (marks[STATE_FRAGMENT].end > 0)
-        _fragment.assign(p + marks[STATE_FRAGMENT].start, marks[STATE_FRAGMENT].end - marks[STATE_FRAGMENT].start, string::COPY);
+        _fragment = uristr.substr(marks[STATE_FRAGMENT].start, marks[STATE_FRAGMENT].end - marks[STATE_FRAGMENT].start);
 
     if (_flags & ALLOW_LEADING_AUTHORITY && !_host.length()) guess_leading_authority();
 
     sync_scheme_info();
 }
 
-static inline void _encode_uri_component_append (const string& src, string& dest, const char* unsafe) {
-    size_t final_size;
-    encode_uri_component(src.data(), src.length(), dest.reserve(dest.length() + src.length()*3) + dest.length(), &final_size, unsafe);
-    dest.resize(dest.length() + final_size);
-}
-
 string URI::to_string (bool relative) const {
     sync_query_string();
     size_t approx_len = _path.length() + _fragment.length() + _qstr.length() + 3;
@@ -250,12 +243,13 @@ string URI::to_string (bool relative) const {
                 str += '@';
             }
 
-            if (_host[0] == '[' && _host[_host.length()-1] == ']') str += _host;
+            const auto& chost = _host;
+            if (chost.front() == '[' && chost.back() == ']') str += _host;
             else _encode_uri_component_append(_host, str, unsafe_host);
 
             if (_port) {
                 str += ':';
-                str += itoa(_port);
+                str += string::from_number(_port);
             }
         }
     }
@@ -282,7 +276,7 @@ void URI::parse_query () const {
     int key_end   = 0;
     int val_start = 0;
     const char delim = _flags & PARAM_DELIM_SEMICOLON ? ';' : '&';
-    const char* str = _qstr.c_str();
+    const char* str = _qstr.data();
     int len = _qstr.length();
     _query.clear();
 
@@ -301,12 +295,13 @@ void URI::parse_query () const {
 
             string key;
             size_t klen = key_end - key_start;
-            if (klen > 0) decode_uri_component(str+key_start, klen, key);
-
-            Query::iterator elem = _query.insert(key, string());
+            if (klen > 0) decode_uri_component(string_view(str+key_start, klen), key);
 
+            string value;
             size_t vlen = i - val_start;
-            if (vlen > 0) decode_uri_component(str+val_start, vlen, elem->second);
+            if (vlen > 0) decode_uri_component(string_view(str+val_start, vlen), value);
+
+            _query.emplace(key, value);
 
             mode = PARSE_MODE_KEY;
             key_start = i+1;
@@ -319,37 +314,32 @@ void URI::parse_query () const {
 void URI::compile_query () const {
     _qstr.clear();
     const char delim = _flags & PARAM_DELIM_SEMICOLON ? ';' : '&';
-    Query::const_iterator begin = _query.cbegin();
-    Query::const_iterator end   = _query.cend();
+    auto begin = _query.cbegin();
+    auto end   = _query.cend();
 
     size_t bufsize = 0;
-    for (Query::const_iterator it = begin; it != end; ++it) {
-        bufsize += (it->first.length() + it->second.length())*3 + 2;
-    }
+    for (auto it = begin; it != end; ++it) bufsize += (it->first.length() + it->second.length())*3 + 2;
     if (bufsize) --bufsize;
 
     _qstr.reserve(bufsize);
 
     char* bufp = _qstr.buf();
     char* ptr = bufp;
-    size_t len;
-    for (Query::const_iterator it = begin; it != end; ++it) {
+    for (auto it = begin; it != end; ++it) {
         if (it != begin) *ptr++ = delim;
-        encode_uri_component(it->first, ptr, &len);
-        ptr += len;
+        ptr += encode_uri_component(it->first, ptr);
         *ptr++ = '=';
-        encode_uri_component(it->second, ptr, &len);
-        ptr += len;
+        ptr += encode_uri_component(it->second, ptr);
     }
-    _qstr.resize(ptr-bufp);
+    _qstr.length(ptr-bufp);
 
     ok_qboth();
 }
 
 void URI::add_query (const Query& addquery) {
     sync_query();
-    Query::const_iterator end = addquery.cend();
-    for (Query::const_iterator it = addquery.cbegin(); it != end; ++it) _query.insert(it->first, it->second);
+    auto end = addquery.cend();
+    for (auto it = addquery.cbegin(); it != end; ++it) _query.emplace(it->first, it->second);
     ok_query();
 }
 
@@ -363,21 +353,41 @@ const std::vector<string> URI::path_segments () const {
     for (size_t i = 0; i < plen; ++i) {
         if (p[i] != '/') continue;
         if (i == start) { start++; continue; }
-        ret.push_back(string());
-        decode_uri_component(p+start, i-start, ret[ret.size()-1]);
+        ret.push_back(decode_uri_component(string_view(p+start, i-start)));
         start = i+1;
     }
-    if (p[plen-1] != '/') ret.push_back(string(p+start, plen-start, string::COPY));
+    if (p[plen-1] != '/') ret.push_back(string(p+start, plen-start));
     return ret;
 }
 
-void URI::path_segments (const std::vector<string>& list) {
-    _path.clear();
-    for (size_t i = 0; i < list.size(); ++i) {
-        if (!list[i].size()) continue;
-        _path += '/';
-        _encode_uri_component_append(list[i], _path, unsafe_path_segment);
+void URI::swap (URI& uri) {
+    std::swap(_scheme,     uri._scheme);
+    std::swap(scheme_info, uri.scheme_info);
+    std::swap(_user_info,  uri._user_info);
+    std::swap(_host,       uri._host);
+    std::swap(_port,       uri._port);
+    std::swap(_path,       uri._path);
+    std::swap(_qstr,       uri._qstr);
+    std::swap(_query,      uri._query);
+    std::swap(_qrev,       uri._qrev);
+    std::swap(_fragment,   uri._fragment);
+    std::swap(_flags,      uri._flags);
+}
+
+void URI::sync_scheme_info () {
+    if (!_scheme) {
+        scheme_info = NULL;
+        return;
     }
+
+    // lowercase the scheme
+    char* p   = _scheme.buf();
+    char* end = p + _scheme.length();
+    for (;p != end; ++p) *p = tolower(*p);
+
+    auto it = scheme_map.find(_scheme);
+    if (it == scheme_map.cend()) scheme_info = NULL;
+    else                         scheme_info = it->second;
 }
 
 }}
