@@ -46,8 +46,8 @@ public:
     static void add_param      (pTHX_ URI* uri, string key, SV* val, bool replace = false);
 
 private:
-    XSURIWrapper (const XSURIWrapper& s) {}
-    XSURIWrapper& operator= (const XSURIWrapper& s) { return *this; }
+    XSURIWrapper (const XSURIWrapper&) {}
+    XSURIWrapper& operator= (const XSURIWrapper&) { return *this; }
 };
 
 class XSURI : public xs::XSBackref, public URI {
@@ -78,8 +78,8 @@ public:
 class XSURI::ftp : public xs::XSBackref, public URI::ftp {
 public:
     ftp () : URI::ftp() {}
-    ftp (const string& source, int flags = 0) : URI::ftp(source, flags), URI::Strict(source, flags) {}
-    ftp (const URI& source)                   : URI::ftp(source),        URI::Strict(source)        {}
+    ftp (const string& source, int flags = 0) : URI::Strict(source, flags), URI::ftp(source, flags) {}
+    ftp (const URI& source)                   : URI::Strict(source),        URI::ftp(source)        {}
 };
 
 }}
